<?php
if(!defined('PLX_ROOT')) { exit; }

/*
 * Add a field for e-mail address in auth.php page for sending a new password.
 * This plugin is using directly mail function from PHP.
 * So, sendmail_path variable must be set in php.ini ( default value is valide ).
 * The new password is stored in $_SESSION array.
 * It is dropped after a while ( see lzLostPasswor::DURATION ).
 * If success, the new password is stored after authentication in users.xml file by plxAdmin::editUsers()
 * */
class kzLostPassword extends plxPlugin {

	const PASSWORD_REQUEST = 'kzLostPassword-request';
	const PASSWORD_MAIL = 'kzLostPassword-mail';
	const DURATION = 1800;

	public function __construct($default_lang) {

		parent::__construct($default_lang);
		parent::setConfigProfil(PROFIL_ADMIN);
		if(defined('PLX_AUTHPAGE')) {
			parent::addHook('AdminAuthPrepend', 'AdminAuthPrepend');
			parent::addHook('AdminAuthTop', 'AdminAuthTop');
			parent::addHook('AdminAuth', 'AdminAuth');
			parent::addHook('AdminAuthEndBody', 'AdminAuthEndBody');
			if(version_compare(PLX_VERSION, '5.6', '<=')) {
				parent::addHook('AdminAuthEndHead', 'AdminAuthEndHead');
			}
		}
	}

	private function __new_password($userInfos, $mailTo, $title='', $url='') {
		$new_password = plxUtils::charAleatoire(16);
		$salt = $userInfos['salt'];
		$_SESSION[__CLASS__ . '-encrypt_pw'] = sha1($salt.md5($new_password));
		$_SESSION[__CLASS__ . '-login'] = $userInfos['login'];
		$_SESSION[__CLASS__ . '-limit'] = time() + self::DURATION;

		$subject = str_replace('#TITLE', $title, self::getLang('MAIL_SUBJECT'));
		$body = wordwrap(
			strtr(self::getLang('BODY_MAIL'), array(
				'#LOGIN'		=> $userInfos['login'],
				'#PASSWORD'		=> $new_password,
				'#IP'			=> $_SERVER['REMOTE_ADDR'],
				'#USER-AGENT'	=> $_SERVER['HTTP_USER_AGENT'],
				'#SITE'			=> $url,
				'#DATE'			=> date(DATE_COOKIE, $_SESSION[__CLASS__ . '-limit'])
			)),
			70
		);
		$headers = array(
			'Content-Type'				=> 'text/plain; charset="' . PLX_CHARSET . '"',
			'MIME-Version'				=> '1.0',
			'Content-transfer-encoding'	=> 'quoted-printable',
			'X-Mailer'					=> 'PHP/' . phpversion()
		);
		return mail($mailTo, $subject, $body,
			version_compare(PHP_VERSION, '7.2', '>=') ?
				$headers :
				implode("\r\n", array_map(function($v, $k) { return "$k: $v"; }, array_values($headers), array_keys($headers)))
		);
	}

	/* ==================== Hooks =========================== */
	public function AdminAuthPrepend() {
		global $plxAdmin;

		$login = filter_input(INPUT_POST, 'login', FILTER_SANITIZE_STRING);
		if(empty($login)) {
			if(filter_input(INPUT_POST, self::PASSWORD_REQUEST, FILTER_VALIDATE_BOOLEAN) === true) {
				$this->error = parent::getLang('INVALIDATE_LOGIN');
			}
			return;
		}

		# A new password is requested and saved in $_SESSION
		if(filter_input(INPUT_POST, self::PASSWORD_REQUEST, FILTER_VALIDATE_BOOLEAN) === true) {
			$mail = filter_input(INPUT_POST, self::PASSWORD_MAIL, FILTER_VALIDATE_EMAIL);
			if(empty($mail)) {
				$this->error = parent::getLang('INVALIDATE_MAIL');
				return;
			}
			foreach($plxAdmin->aUsers as $userId=>$userInfos) {
				if(
					!empty($userInfos['active']) and
					empty($userInfos['delete']) and
					!empty($userInfos['email']) and
					$userInfos['login'] == $login and
					$userInfos['email'] == $mail
				) {
					if(self::__new_password($userInfos, $mail, $plxAdmin->aConf['title'], $plxAdmin->racine)) {
						$this->success = true;
						$this->error = parent::getLang('SENT_PASSWORD');
						if(!empty($_SESSION['maxtry'])) { unset($_SESSION['maxtry']); }
						@error_log('[kzLostPassword plugin] New password required from ' . $_SERVER['REMOTE_ADDR'] . 'and sent to ' . $mail);
					} else {
						$this->error = parent::getLang('FAILURE_MAIL');
					}
					$plxAdmin->aUsers[$userId]['active'] = false;
					return;
					break;
				}
			}
			$this->error = parent::getLang('UNKNOWN_LOGIN_MAIL');
			return;
		}

		# Checks if the new password is expired
		if(
			!empty($_SESSION[__CLASS__ . '-limit']) and
			$_SESSION[__CLASS__ . '-limit'] < time()
		) {
			unset($_SESSION[__CLASS__ . '-limit']);
			unset($_SESSION[__CLASS__ . '-encrypt_pw']);
			unset($_SESSION[__CLASS__ . '-login']);
			return;
		}

		# Checks if the new password is using
		if(
			!empty($_SESSION[__CLASS__ . '-encrypt_pw']) and
			!empty($_SESSION[__CLASS__ . '-login'])
		) {
			$password = filter_input(INPUT_POST, 'password', FILTER_SANITIZE_STRING);
			if(!is_string($password)) { return; }
			$login = filter_input(INPUT_POST, 'login', FILTER_SANITIZE_STRING);
			if(!is_string($login) or $_SESSION[__CLASS__ . '-login'] != $login) { return; }
			foreach($plxAdmin->aUsers as $userId=>$infos) {
				if(
					!empty($infos['active']) and
					empty($infos['delete']) and
					$infos['login'] == $login
				) {
					$salt = $infos['salt'];
					$encrypt_pw = sha1($salt . md5($password));
					if($encrypt_pw === $infos['password']) {
						# The old password is using
						unset($_SESSION[__CLASS__ . '-limit']);
						unset($_SESSION[__CLASS__ . '-encrypt_pw']);
						unset($_SESSION[__CLASS__ . '-login']);
						return;
						break;
					}
					# second chance
					if($encrypt_pw === $_SESSION[__CLASS__ . '-encrypt_pw']) {
						# The new password is saved into users.xml file
						$plxAdmin->aUsers[$userId]['password'] = $encrypt_pw;
						$result = $plxAdmin->editUsers(null, true);
						return;
						break;
					}
				}
			}
		}
	}

	public function AdminAuthTop() {
		if(!empty($this->error)) {
			echo '<?php ' . PHP_EOL;
?>
$msg = '<?= $this->error ?>';
$error = '<?= (!empty($this->success)) ? 'success' : 'error' ?>';
<?php
			echo ' ?>' . PHP_EOL;
		}
	}

	public function AdminAuth() {
?>
						<div class="grid">
							<div class="col sml-12 <?php echo __CLASS__; ?>">
								<label for="id_<?php echo self::PASSWORD_REQUEST; ?>" class="text-center"><?php echo self::lang('LOST_PASSWORD'); ?> ?</label>
								<input type="checkbox" name="<?php echo self::PASSWORD_REQUEST; ?>" id="id_<?php echo self::PASSWORD_REQUEST; ?>" value="1" />
								<i class="ico icon-mail">
<?php readfile(__DIR__ . '/icon-mail.svg'); ?>
								</i>
								<input type="email" name="<?php echo self::PASSWORD_MAIL; ?>" id="id_<?php echo self::PASSWORD_MAIL; ?>" placeholder="<?php echo $this->lang('AUTH_PLACEHOLDER'); ?>" maxlength="30" autocomplete="email" inputmode="email" class="full-width" autocomplete="off" />
							</div>
						</div>
<?php
	}

	public function AdminAuthEndBody() {
?>
	<script type="text/javascript">
		(function() {
			'use strict';
			const login = document.getElementById('id_login');

			login.required = true;
		})();
	</script>
<?php
	}

	public function AdminAuthEndHead() {
		$href = PLX_PLUGINS . __CLASS__ . '/css/admin.css';
?>
	<link rel="stylesheet" type="text/css" href="<?= $href ?>" media="screen" />
<?php
	}
}
?>
